use crate::{config::ServerConfig, server::Server};
use std::sync::Arc;

pub struct ServerBuilder {
    pub config: ServerConfig,
}

impl ServerBuilder {
    pub fn address(mut self, ip: &str, port: u16) -> Self {
        self.config.ip = ip.to_string();
        self.config.port = port;
        self
    }

    pub fn private_ips(mut self) -> Self {
        self.config.ips_filter = true;

        self.config.private_ips = true;
        self
    }

    pub fn ips(mut self, ips: Vec<String>) -> Self {
        self.config.ips_filter = true;

        self.config.ips = ips;
        self
    }

    pub fn api_key(mut self, api_key: &str) -> Self {
        self.config.api_key = Some(api_key.to_string());
        self
    }

    pub fn build(self) -> Server {
        Server {
            config: Arc::new(self.config),
        }
    }
}
