use http::Request;
use hyper::body::Incoming;
use std::net::IpAddr;

pub struct ServerConfig {
    pub ip: String,
    pub port: u16,

    pub ips_filter: bool,
    pub private_ips: bool,
    pub ips: Vec<String>,

    pub api_key: Option<String>,
}

impl ServerConfig {
    pub fn is_ip_authorized(&self, ip: &IpAddr) -> bool {
        if !self.ips_filter {
            return true;
        }

        if self.private_ips {
            let is_private = match ip {
                IpAddr::V4(ip4) => ip4.is_private(),
                IpAddr::V6(_) => false,
            };

            if is_private {
                return true;
            }
        }

        let ips = &self.ips;
        if ips.is_empty() {
            return true;
        }

        ips.iter()
            .any(|authorized_ip| &ip.to_string() == authorized_ip)
    }

    pub fn is_req_authorized(&self, req: &Request<Incoming>) -> bool {
        if self.api_key.is_none() {
            return true;
        }

        match req.headers().get("X-API-Key") {
            Some(header) => header.eq(self.api_key.as_ref().unwrap()),
            None => false,
        }
    }
}

impl Default for ServerConfig {
    fn default() -> Self {
        ServerConfig {
            ip: "127.0.0.1".to_string(),
            port: 8080,

            ips_filter: false,
            private_ips: false,
            ips: Vec::new(),

            api_key: None,
        }
    }
}
